@@ -105,6 +105,8 @@ struct NautilusDirectoryDetails
 	LinkInfoReadState *link_info_read_state;
 
 	GList *file_operations_in_progress; /* list of FileOperation * */
+
+	GHashTable *hidden_file_hash;
 };
 
 /* A request for information about one or more files. */
@@ -217,5 +219,9 @@ void               nautilus_directory_add_file_to_work_queue          (NautilusD
 void               nautilus_directory_remove_file_from_work_queue     (NautilusDirectory *directory,
 								       NautilusFile *file);
 
+/* KDE compatibility hacks */
+
+void               nautilus_set_kde_trash_name                        (const char *trash_dir);
+
 /* debugging functions */
 int                nautilus_directory_number_outstanding              (void);
