@@ -46,6 +46,9 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include "nautilus-file-changes-queue.h"
+#include "nautilus-file-private.h"
+#include "nautilus-desktop-icon-file.h"
+#include "nautilus-desktop-link-monitor.h"
 #include "nautilus-global-preferences.h"
 #include "nautilus-link.h"
 #include "nautilus-trash-monitor.h"
@@ -117,10 +120,12 @@ typedef struct {
 	int last_icon_position_index;
 	GList *uris;
 	const GList *last_uri;
+	int screen;
 } IconPositionIterator;
 
 static IconPositionIterator *
-icon_position_iterator_new (GArray *icon_positions, const GList *uris)
+icon_position_iterator_new (GArray *icon_positions, const GList *uris,
+			    int screen)
 {
 	IconPositionIterator *result;
 	guint index;
@@ -137,6 +142,7 @@ icon_position_iterator_new (GArray *icon_positions, const GList *uris)
 
 	result->uris = eel_g_str_list_copy ((GList *)uris);
 	result->last_uri = result->uris;
+	result->screen = screen;
 
 	return result;
 }
@@ -497,6 +503,7 @@ typedef enum {
 	ERROR_NOT_WRITABLE,
 	ERROR_NOT_ENOUGH_PERMISSIONS,
 	ERROR_NO_SPACE,
+	ERROR_SOURCE_IN_TARGET,
 	ERROR_OTHER
 } NautilusFileOperationsErrorKind;
 
@@ -581,10 +588,19 @@ build_error_string (const char *source_name, const char *target_name,
 		 */
 		switch (operation_kind) {
 		case TRANSFER_MOVE:
-			if (error_kind == ERROR_NOT_ENOUGH_PERMISSIONS) {
+			switch (error_kind) {
+			case ERROR_NOT_ENOUGH_PERMISSIONS:
 				error_string = _("Error while moving.\n\n"
 						 "\"%s\" cannot be moved because you do not have "
 						 "permissions to change it or its parent folder.");
+				break;
+			case ERROR_SOURCE_IN_TARGET:
+				error_string = _("Error while moving.\n\n"
+						 "\"%s\" cannot be moved because it or its parent folder "
+						 "are contained in the destination.");
+				break;
+			default:
+				break;
 			}
 			break;
 		case TRANSFER_MOVE_TO_TRASH:
@@ -787,7 +803,7 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 	switch (transfer_info->error_mode) {
 	case GNOME_VFS_XFER_ERROR_MODE_QUERY:
 
-		/* transfer error, prompt the user to continue or stop */
+		/* transfer error, prompt the user to continue or cancel */
 
 		/* stop timeout while waiting for user */
 		nautilus_file_operations_progress_pause_timeout (transfer_info->progress_dialog);
@@ -862,6 +878,10 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 		} else if (progress_info->vfs_status == GNOME_VFS_ERROR_NO_SPACE) {
 			error_location = ERROR_LOCATION_TARGET;
 			error_kind = ERROR_NO_SPACE;
+		} else if (progress_info->vfs_status == GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY
+			   && transfer_info->kind == TRANSFER_MOVE) {
+			error_location = ERROR_LOCATION_SOURCE_OR_PARENT;
+			error_kind = ERROR_SOURCE_IN_TARGET;
 		}
 
 		text = build_error_string (formatted_source_name, formatted_target_name,
@@ -890,10 +910,11 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 			break;
 		}
 
-		if (error_location == ERROR_LOCATION_TARGET) {
+		if (error_location == ERROR_LOCATION_TARGET ||
+		    error_kind == ERROR_SOURCE_IN_TARGET) {
 			/* We can't continue, just tell the user. */
 			eel_run_simple_dialog (parent_for_error_dialog (transfer_info),
-				TRUE, text, dialog_title, _("Stop"), NULL);
+				TRUE, text, dialog_title, GTK_STOCK_OK, NULL);
 			error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 
 		} else if ((error_location == ERROR_LOCATION_SOURCE
@@ -908,16 +929,16 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 			error_dialog_button_pressed = eel_run_simple_dialog
 				(parent_for_error_dialog (transfer_info), TRUE, text, 
 				dialog_title,
-				 _("Skip"), _("Stop"), NULL);
+				 GTK_STOCK_CANCEL, _("Skip"), NULL);
 				 
 			switch (error_dialog_button_pressed) {
 			case 0:
-				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_SKIP;
-				break;
-			case 1:
 			case GTK_RESPONSE_DELETE_EVENT:
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 				break;
+			case 1:
+				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_SKIP;
+				break;
 			default:
 				g_assert_not_reached ();
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
@@ -928,17 +949,17 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 			error_dialog_button_pressed = eel_run_simple_dialog
 				(parent_for_error_dialog (transfer_info), TRUE, text, 
 				 dialog_title,
-				 _("Skip"), _("Retry"), _("Stop"), NULL);
+				 _("Skip"), GTK_STOCK_CANCEL, _("Retry"), NULL);
 
 			switch (error_dialog_button_pressed) {
 			case 0:
 				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_SKIP;
 				break;
 			case 1:
-				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_RETRY;
+				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
 				break;
 			case 2:
-				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_ABORT;
+				error_dialog_result = GNOME_VFS_XFER_ERROR_ACTION_RETRY;
 				break;
 			default:
 				g_assert_not_reached ();
@@ -973,17 +994,8 @@ handle_transfer_vfs_error (const GnomeVFSXferProgressInfo *progress_info,
 static gboolean
 is_special_link (const char *uri)
 {
-	char *local_path;
-	gboolean is_special;
 
-	local_path = gnome_vfs_get_local_path_from_uri (uri);
-	if (local_path == NULL) {
-		return FALSE;
-	}
-	is_special = nautilus_link_local_is_special_link (local_path);
-	g_free (local_path);
-
-	return is_special;
+	return eel_uri_is_desktop (uri);
 }
 
 static int
@@ -1085,15 +1097,21 @@ get_link_name (char *name, int count)
 {
 	char *result;
 	char *unescaped_name;
+	char *unescaped_tmp_name;
 	char *unescaped_result;
+	char *new_file;
 
 	const char *format;
 	
 	g_assert (name != NULL);
 
-	unescaped_name = gnome_vfs_unescape_string (name, "/");
+	unescaped_tmp_name = gnome_vfs_unescape_string (name, "/");
 	g_free (name);
 
+	unescaped_name = g_filename_to_utf8 (unescaped_tmp_name, -1,
+					     NULL, NULL, NULL);
+	g_free (unescaped_tmp_name);
+
 	if (count < 1) {
 		g_warning ("bad count in get_link_name");
 		count = 1;
@@ -1146,11 +1164,12 @@ get_link_name (char *name, int count)
 		}
 		unescaped_result = g_strdup_printf (format, count, unescaped_name);
 	}
-
-	result = gnome_vfs_escape_path_string (unescaped_result);
+	new_file = g_filename_from_utf8 (unescaped_result, -1, NULL, NULL, NULL);
+	result = gnome_vfs_escape_path_string (new_file);
 	
 	g_free (unescaped_name);
 	g_free (unescaped_result);
+	g_free (new_file);
 
 	return result;
 }
@@ -1437,18 +1456,25 @@ static char *
 get_next_duplicate_name (char *name, int count_increment)
 {
 	char *unescaped_name;
+	char *unescaped_tmp_name;
 	char *unescaped_result;
 	char *result;
+	char *new_file;
 
-	unescaped_name = gnome_vfs_unescape_string (name, "/");
+	unescaped_tmp_name = gnome_vfs_unescape_string (name, "/");
 	g_free (name);
 
+	unescaped_name = g_filename_to_utf8 (unescaped_tmp_name, -1,
+					     NULL, NULL, NULL);
+	g_free (unescaped_tmp_name);
+
 	unescaped_result = get_duplicate_name (unescaped_name, count_increment);
 	g_free (unescaped_name);
 
-	result = gnome_vfs_escape_path_string (unescaped_result);
+	new_file = g_filename_from_utf8 (unescaped_result, -1, NULL, NULL, NULL);
+	result = gnome_vfs_escape_path_string (new_file);
 	g_free (unescaped_result);
-	
+	g_free (new_file);
 	return result;
 }
 
@@ -1510,7 +1536,7 @@ apply_one_position (IconPositionIterator *position_iterator,
 	GdkPoint point;
 
 	if (icon_position_iterator_get_next (position_iterator, source_name, &point)) {
-		nautilus_file_changes_queue_schedule_position_set (target_name, point);
+		nautilus_file_changes_queue_schedule_position_set (target_name, point, position_iterator->screen);
 	} else {
 		nautilus_file_changes_queue_schedule_position_remove (target_name);
 	}
@@ -1659,19 +1685,6 @@ append_basename (const GnomeVFSURI *target_directory,
 	return gnome_vfs_uri_dup (target_directory);
 }
 
-static gboolean 
-vfs_uri_is_special_link (GnomeVFSURI *vfs_uri)
-{
-	char *uri;
-	gboolean is_special;
-
-	uri = gnome_vfs_uri_to_string (vfs_uri, GNOME_VFS_URI_HIDE_NONE);
-	is_special = is_special_link (uri);
-	g_free (uri);
-
-	return is_special;
-}
-
 void
 nautilus_file_operations_copy_move (const GList *item_uris,
 				    GArray *relative_item_points,
@@ -1693,11 +1706,15 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	SyncTransferInfo *sync_transfer_info;
 	GnomeVFSResult result;
 	gboolean target_is_trash;
-	gboolean is_desktop_trash_link;
 	gboolean duplicate;
+	gboolean target_is_mapping;
+	gboolean have_nonlocal_source;
 	
 	IconPositionIterator *icon_position_iterator;
 
+	GdkScreen *screen;
+	int screen_num;
+
 	g_assert (item_uris != NULL);
 
 	target_dir_uri = NULL;
@@ -1705,12 +1722,17 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	result = GNOME_VFS_OK;
 
 	target_is_trash = FALSE;
+	target_is_mapping = FALSE;
 	if (target_dir != NULL) {
 		if (eel_uri_is_trash (target_dir)) {
 			target_is_trash = TRUE;
 		} else {
 			target_dir_uri = gnome_vfs_uri_new (target_dir);
 		}
+		if (strncmp (target_dir, "burn:", 5) == 0) {
+			target_is_mapping = TRUE;
+		}
+			
 	}
 
 	/* Build the source and target URI lists and figure out if all
@@ -1718,6 +1740,7 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	 */
 	source_uri_list = NULL;
 	target_uri_list = NULL;
+	have_nonlocal_source = FALSE;
 	duplicate = copy_action != GDK_ACTION_MOVE;
 	for (p = item_uris; p != NULL; p = p->next) {
 		/* Filter out special Nautilus link files */
@@ -1734,6 +1757,11 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 		if (source_uri == NULL) {
 			continue;
 		}
+		
+		if (strcmp (source_uri->method_string, "file") != 0) {
+			have_nonlocal_source = TRUE;
+		}
+			
 		source_dir_uri = gnome_vfs_uri_get_parent (source_uri);
 		target_uri = NULL;
 		if (target_dir != NULL) {
@@ -1794,8 +1822,10 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	source_uri_list = g_list_reverse (source_uri_list);
 	target_uri_list = g_list_reverse (target_uri_list);
 
-
-	if (copy_action == GDK_ACTION_MOVE) {
+	if (target_is_mapping && !have_nonlocal_source && (copy_action == GDK_ACTION_COPY || copy_action == GDK_ACTION_MOVE)) {
+		copy_action = GDK_ACTION_LINK;
+	}
+	if (copy_action == GDK_ACTION_MOVE && !target_is_mapping) {
 		move_options |= GNOME_VFS_XFER_REMOVESOURCE;
 	} else if (copy_action == GDK_ACTION_LINK) {
 		move_options |= GNOME_VFS_XFER_LINK_ITEMS;
@@ -1804,11 +1834,13 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 	/* set up the copy/move parameters */
 	transfer_info = transfer_info_new (parent_view);
 	if (relative_item_points != NULL && relative_item_points->len > 0) {
+		screen = gtk_widget_get_screen (GTK_WIDGET (parent_view));
+		screen_num = gdk_screen_get_number (screen);
 		/* FIXME: we probably don't need an icon_position_iterator
 		 * here at all.
 		 */
 		icon_position_iterator = icon_position_iterator_new
-			(relative_item_points, item_uris);
+			(relative_item_points, item_uris, screen_num);
 	} else {
 		icon_position_iterator = NULL;
 	}
@@ -1893,21 +1925,13 @@ nautilus_file_operations_copy_move (const GList *item_uris,
 			    	/* Distinguish Trash file on desktop from other trash folders for
 			    	 * message purposes.
 			    	 */
-				/* FIXME: is_special_link finds more than just trash links,
-				 * so these messages are wrong. 
-				 */
-			    	is_desktop_trash_link = vfs_uri_is_special_link (uri);
 
 				eel_run_simple_dialog
 					(parent_view,
 					 FALSE,
 					 ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0)
-						 ? (is_desktop_trash_link
-						    ? _("The Trash must remain on the desktop.")
-						    : _("You cannot move this trash folder."))
-						 : (is_desktop_trash_link
-						    ? _("You cannot copy the Trash.")
-						    : _("You cannot copy this trash folder.")),
+						 ? _("You cannot move this trash folder.")
+						 : _("You cannot copy this trash folder."),
 					 ((move_options & GNOME_VFS_XFER_REMOVESOURCE) != 0)
 						 ? _("Can't Change Trash Location")
 						 : _("Can't Copy Trash"),
@@ -2013,7 +2037,7 @@ handle_new_folder_vfs_error (const GnomeVFSXferProgressInfo *progress_info, NewF
 	}
 	
 	eel_show_error_dialog (error_string, _("Error creating new folder"),
-				    GTK_WINDOW (state->parent_view));
+				    GTK_WINDOW (gtk_widget_get_toplevel (state->parent_view)));
 	
 	g_free (error_string_to_free);
 	
@@ -2088,6 +2112,7 @@ nautilus_file_operations_new_folder (GtkWidget *parent_view,
 {
 	GList *target_uri_list;
 	GnomeVFSURI *uri, *parent_uri;
+	char *dirname;
 	NewFolderTransferState *state;
 
 	state = g_new (NewFolderTransferState, 1);
@@ -2099,7 +2124,10 @@ nautilus_file_operations_new_folder (GtkWidget *parent_view,
 	/* pass in the target directory and the new folder name as a destination URI */
 	parent_uri = gnome_vfs_uri_new (parent_dir);
 	/* localizers: the initial name of a new folder  */
-	uri = gnome_vfs_uri_append_file_name (parent_uri, _("untitled folder"));
+
+	dirname = g_filename_from_utf8 (_("untitled folder"), -1, NULL, NULL, NULL);
+	uri = gnome_vfs_uri_append_file_name (parent_uri, dirname);
+	g_free (dirname);
 	target_uri_list = g_list_prepend (NULL, uri);
 	
 	gnome_vfs_async_xfer (&state->handle, NULL, target_uri_list,
@@ -2120,15 +2148,41 @@ nautilus_file_operations_delete (const GList *item_uris,
 {
 	GList *uri_list;
 	const GList *p;
+	const char *item_uri;
+	NautilusFile *file;
 	TransferInfo *transfer_info;
 
 	uri_list = NULL;
 	for (p = item_uris; p != NULL; p = p->next) {
-		uri_list = g_list_prepend (uri_list, 
-					   gnome_vfs_uri_new ((const char *) p->data));
+		item_uri = (const char *) p->data;
+
+		if (eel_uri_is_desktop (item_uri)) {
+			file = nautilus_file_get_existing (item_uri);
+			if (file != NULL) {
+				if (NAUTILUS_IS_DESKTOP_ICON_FILE (file)) {
+					NautilusDesktopLink *link;
+
+					link = nautilus_desktop_icon_file_get_link (NAUTILUS_DESKTOP_ICON_FILE (file));
+
+					nautilus_desktop_link_monitor_delete_link (nautilus_desktop_link_monitor_get (),
+										   link,
+										   parent_view);
+					
+					g_object_unref (link);
+				}
+				nautilus_file_unref (file);
+			}
+		} else {
+			uri_list = g_list_prepend (uri_list, 
+						   gnome_vfs_uri_new (item_uri));
+		}
 	}
 	uri_list = g_list_reverse (uri_list);
 
+	if (uri_list == NULL) {
+		return;
+	}
+
 	transfer_info = transfer_info_new (parent_view);
 
 	/* localizers: progress dialog title */
