@@ -50,6 +50,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-messagebox.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <pthread.h>
 #include <stdio.h>
@@ -61,17 +62,34 @@
 
 #ifdef HAVE_SYS_VFSTAB_H
 #include <sys/vfstab.h>
-#else
+#elif HAVE_FSTAB_H
 #include <fstab.h>
 #endif
 
+#if defined(HAVE_SYS_MNTCTL_H) && defined(HAVE_SYS_VMOUNT_H) && defined(HAVE_SYS_VFS_H)
+
+#include <sys/mntctl.h>
+#include <sys/vfs.h>
+#include <sys/vmount.h>
+#include <fshelp.h>
+
+#define AIX_MNT 1
+
+#endif /* HAVE_SYS_MNTCTL_H && HAVE_SYS_VMOUNT_H && HAVE_SYS_VFS_H*/
+
 #ifdef HAVE_MNTENT_H
 #include <mntent.h>
+#ifndef __CYGWIN__
 #define MOUNT_TABLE_PATH _PATH_MNTTAB
+#else
+#define MOUNT_TABLE_PATH MOUNTED
+#endif
 #elif defined (HAVE_SYS_MNTTAB_H)
 #define SOLARIS_MNT 1
 #include <sys/mnttab.h>
 #define MOUNT_TABLE_PATH "/etc/mnttab"
+#elif defined (AIX_MNT)
+#define MOUNT_TABLE_PATH "/etc/filesystems"
 #else
 /* FIXME: How does this help anything? */
 #define MOUNT_TABLE_PATH ""
@@ -106,6 +124,9 @@
 #ifndef HAVE_SETMNTENT
 #define setmntent(f,m) fopen(f,m)
 #endif
+#ifndef HAVE_ENDMNTENT
+#define endmntent(f) fclose(f)
+#endif
 
 #ifdef HAVE_CDDA_INTERFACE_H
 #ifdef HAVE_CDDA_PARANOIA_H
@@ -142,6 +163,13 @@ typedef struct mnttab MountTableEntry;
 #elif defined (HAVE_GETMNTINFO)
 typedef struct statfs MountTableEntry;
 #define MOUNT_TABLE_ENTRY_TYPE(ent) ((ent)->f_fstypename)
+#elif defined(AIX_MNT)
+typedef struct {
+        char mnt_mount[PATH_MAX];
+        char mnt_special[PATH_MAX];
+        char mnt_fstype[16];
+        char mnt_options[128];
+} MountTableEntry;
 #else
 typedef struct mntent MountTableEntry;
 #define MOUNT_TABLE_ENTRY_TYPE(ent) ((ent)->mnt_type)
@@ -169,6 +197,8 @@ struct NautilusVolume {
 
 struct NautilusVolumeMonitorDetails
 {
+	GnomeVFSMonitorHandle *monitor_handle;
+
 	GList *mounts;
 	GList *removable_volumes;
 	guint mount_volume_timer_id;
@@ -191,6 +221,7 @@ enum {
 	VOLUME_UNMOUNT_STARTED,
 	VOLUME_UNMOUNT_FAILED,
 	VOLUME_UNMOUNTED,
+	REMOVABLE_VOLUMES_CHANGED,
 	LAST_SIGNAL
 };
 static guint signals[LAST_SIGNAL];
@@ -198,6 +229,7 @@ static guint signals[LAST_SIGNAL];
 
 static void            nautilus_volume_monitor_init       (NautilusVolumeMonitor      *desktop_mounter);
 static void            nautilus_volume_monitor_class_init (NautilusVolumeMonitorClass *klass);
+static void            nautilus_volume_monitor_dispose          (GObject                    *object);
 static void            nautilus_volume_monitor_finalize         (GObject                    *object);
 static char *          get_iso9660_volume_name                  (NautilusVolume             *volume,
 								 int                         volume_fd);
@@ -224,6 +256,8 @@ static void            nautilus_file_system_type_free           (NautilusFileSys
 static gboolean        entry_is_supermounted_volume             (const MountTableEntry *ent, 
 								 const NautilusVolume *volume);
 
+static void            verify_current_mount_state               (NautilusVolumeMonitor *monitor);
+
 #ifdef HAVE_CDDA
 static gboolean        locate_audio_cd                          (void);
 #endif
@@ -284,6 +318,32 @@ load_file_system_table (void)
 	return table;
 }
 
+static void
+update_removable_volumes (NautilusVolumeMonitor *monitor)
+{
+	free_mount_list (monitor->details->removable_volumes);
+
+	monitor->details->removable_volumes = get_removable_volumes (monitor);
+
+	g_signal_emit (monitor, signals[REMOVABLE_VOLUMES_CHANGED], 0);
+
+	verify_current_mount_state (monitor);	
+}
+
+static void
+fstab_monitor_callback (GnomeVFSMonitorHandle *handle,
+			const char *monitor_uri,
+			const char *info_uri,
+			GnomeVFSMonitorEventType event_type,
+			gpointer user_data)
+{
+	NautilusVolumeMonitor *monitor;
+
+	monitor = NAUTILUS_VOLUME_MONITOR (user_data);
+
+	update_removable_volumes (monitor);
+}
+
 static void
 nautilus_volume_monitor_init (NautilusVolumeMonitor *monitor)
 {
@@ -293,6 +353,12 @@ nautilus_volume_monitor_init (NautilusVolumeMonitor *monitor)
 	monitor->details->file_system_table = load_file_system_table ();
 	monitor->details->removable_volumes = get_removable_volumes (monitor);
 	find_volumes (monitor);
+
+	gnome_vfs_monitor_add (&monitor->details->monitor_handle,
+			       "file:///etc/fstab",
+			       GNOME_VFS_MONITOR_FILE,
+			       fstab_monitor_callback,
+			       monitor);
 }
 
 static void
@@ -302,6 +368,7 @@ nautilus_volume_monitor_class_init (NautilusVolumeMonitorClass *klass)
 
 	object_class = G_OBJECT_CLASS (klass);
 
+	object_class->dispose = nautilus_volume_monitor_dispose;
 	object_class->finalize = nautilus_volume_monitor_finalize;
 
 	signals[VOLUME_MOUNTED] 
@@ -344,6 +411,16 @@ nautilus_volume_monitor_class_init (NautilusVolumeMonitorClass *klass)
 		                g_cclosure_marshal_VOID__POINTER,
 		                G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+	signals[REMOVABLE_VOLUMES_CHANGED] 
+		= g_signal_new ("removable_volumes_changed",
+		                G_TYPE_FROM_CLASS (object_class),
+		                G_SIGNAL_RUN_LAST,
+		                G_STRUCT_OFFSET (NautilusVolumeMonitorClass, 
+						     removable_volumes_changed),
+		                NULL, NULL,
+		                g_cclosure_marshal_VOID__VOID,
+		                G_TYPE_NONE, 0);
+
 	/* Check environment a bit. */
 	if (g_file_test ("/vol/dev", G_FILE_TEST_EXISTS)) {
 		floppy_device_path_prefix = "/vol/dev/diskette/";
@@ -359,6 +436,21 @@ nautilus_volume_monitor_class_init (NautilusVolumeMonitorClass *klass)
 				     G_FILE_TEST_EXISTS);
 }
 
+static void
+nautilus_volume_monitor_dispose (GObject *object)
+{
+	NautilusVolumeMonitor *monitor;
+	
+	monitor = NAUTILUS_VOLUME_MONITOR (object);
+
+	if (monitor->details->monitor_handle) {
+		gnome_vfs_monitor_cancel (monitor->details->monitor_handle);
+		monitor->details->monitor_handle = NULL;
+	}
+
+	EEL_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
+}
+
 static void
 nautilus_volume_monitor_finalize (GObject *object)
 {
@@ -425,6 +517,7 @@ floppy_sort (const NautilusVolume *volume1, const NautilusVolume *volume2)
 	if (!is_floppy_1 && is_floppy_2) {
 		return +1;
 	}
+
 	return 0;
 }
 
@@ -513,6 +606,124 @@ has_removable_mntent_options (MountTableEntry *ent)
 
 #endif
 
+#if defined(AIX_MNT)
+
+/*
+ * functions to parse /etc/filesystems
+ *
+ */
+
+/* aix_fsprop_getc
+ *
+ * read character, ignoring comments (begin with '*', end with '\n'
+ */
+static int
+aix_fs_getc (FILE *fd)
+{
+	int c;
+
+	while ((c = getc (fd)) == '*') {
+		while (((c = getc (fd)) != '\n') && (c != EOF)) {
+		}
+	}
+}
+
+/* aix_fs_ignorespace
+ *
+ * eat all continuous spaces in a file
+ *
+ */
+
+static int
+aix_fs_ignorespace (FILE *fd)
+{
+	int c;
+	
+	while ((c = aix_fs_getc (fd)) != EOF) {
+		if (!g_ascii_isspace (c)) {
+			ungetc (c,fd);
+			return c;
+		}
+	}
+
+        return EOF;
+}
+
+/* aix_fs_getword
+ *
+ * read one word from file
+ */
+
+static int
+aix_fs_getword (FILE *fd, char *word)
+{
+	int c;
+
+	aix_fs_ignorespace (fd);
+
+	while (((c = aix_fs_getc (fd)) != EOF) && !g_ascii_isspace (c))	{
+		if (c == '"') {
+			while (((c = aix_fs_getc (fd)) != EOF) && (c != '"')) {
+				*word++ = c;
+			}
+		} else {
+			*word++ = c;
+		}
+	}
+	*word = 0;
+
+	return c;
+}
+
+/* aix_fs_get
+ *
+ * read mount points properties
+ */
+
+static int
+aix_fs_get (FILE *fd, MountTableEntry *prop)
+{
+	static char word[PATH_MAX] = { 0 };
+	char value[PATH_MAX];
+
+	/* read stanza */
+
+	if (word[0] == 0) {
+		if (aix_fs_getword (fd, word) == EOF)
+			return EOF;
+	}
+
+	word[strlen(word) - 1] = 0;
+	strcpy (prop->mnt_mount, word);
+
+	/* read attributes and value */
+
+	while (aix_fs_getword (fd, word) != EOF) {
+		/* test if is attribute or new stanza */
+
+		if (word[strlen(word) - 1] == ':') {
+			return 0;
+		}
+
+		/* read "=" */
+		aix_fs_getword (fd, value);
+
+		/* read value */
+		aix_fs_getword (fd, value);
+
+		if (strcmp (word, "dev") == 0) {
+			strcpy (prop->mnt_special, value);
+		} else if (strcmp (word, "vfs") == 0) {
+			strcpy (prop->mnt_fstype, value);
+		} else if (strcmp (word, "options") == 0) {
+			strcpy(prop->mnt_options, value);
+		}
+	}
+
+	return 0;
+}
+#endif
+
 /* get_removable_volumes
  *	
  * Returns a list a device paths.
@@ -527,7 +738,7 @@ get_removable_volumes (NautilusVolumeMonitor *monitor)
 	MountTableEntry *ent;
 	NautilusVolume *volume;
 	char * fs_opt;
-#ifdef HAVE_SYS_MNTTAB_H
+#if defined(HAVE_SYS_MNTTAB_H) || defined(AIX_MNT)
         MountTableEntry ent_storage;
 #endif
 #ifdef HAVE_GETMNTINFO
@@ -556,7 +767,17 @@ get_removable_volumes (NautilusVolumeMonitor *monitor)
 		return NULL;
 	}
 	
-#ifdef HAVE_SYS_MNTTAB_H
+#if defined(AIX_MNT)
+        ent = &ent_storage;
+	while (!aix_fs_get (file, ent)) {
+		if (strcmp ("cdrfs", ent->mnt_fstype) == 0) {
+			volume = create_volume (ent->mnt_special, ent->mnt_mount);
+			volume->is_removable = TRUE;
+			volumes = finish_creating_volume_and_prepend
+				(monitor, volume, ent->mnt_fstype, volumes);
+		}
+	}
+#elif defined(HAVE_SYS_MNTTAB_H)
         ent = &ent_storage;
 	while (! getmntent (file, ent)) {
 		if (eel_str_has_prefix (ent->mnt_special, noauto_string)) {
@@ -589,7 +810,7 @@ get_removable_volumes (NautilusVolumeMonitor *monitor)
 	}
 #endif
 			
-	fclose (file);
+	endmntent (file);
 	
 #ifdef HAVE_CDDA
 	volume = create_volume (CD_AUDIO_PATH, CD_AUDIO_PATH);
@@ -663,7 +884,7 @@ volume_is_removable (const NautilusVolume *volume)
 	}
 #endif
 	
-	fclose (file);
+	endmntent (file);
 	return removable;
 }
 
@@ -679,6 +900,52 @@ nautilus_volume_get_name (const NautilusVolume *volume)
 }
 
 
+char *
+nautilus_volume_get_icon (const NautilusVolume *volume)
+{
+	char *icon_name;
+	
+	icon_name = "gnome-dev-harddisk";
+	switch (nautilus_volume_get_device_type (volume)) {
+	case NAUTILUS_DEVICE_AUDIO_CD:
+	case NAUTILUS_DEVICE_CDROM_DRIVE:
+		icon_name = "gnome-dev-cdrom";
+		break;
+
+	case NAUTILUS_DEVICE_FLOPPY_DRIVE:
+		icon_name = "gnome-dev-floppy";
+		break;
+
+	case NAUTILUS_DEVICE_JAZ_DRIVE:
+		icon_name = "gnome-dev-jazdisk";
+		break;
+
+	case NAUTILUS_DEVICE_MEMORY_STICK:
+		icon_name = "gnome-dev-memory";
+		break;
+	
+	case NAUTILUS_DEVICE_NFS:
+		icon_name = "gnome-fs-nfs";
+		break;
+
+	case NAUTILUS_DEVICE_SMB:
+		icon_name = "gnome-fs-smb";
+		break;
+	
+	case NAUTILUS_DEVICE_ZIP_DRIVE:
+		icon_name = "gnome-dev-zipdisk";
+		break;
+
+	case NAUTILUS_DEVICE_APPLE:
+	case NAUTILUS_DEVICE_WINDOWS:
+	case NAUTILUS_DEVICE_CAMERA:
+	case NAUTILUS_DEVICE_UNKNOWN:
+		break;
+	}
+
+	return g_strdup (icon_name);
+}
+
 /* modify_volume_name_for_display
  *	
  * Modify volume to be in human readable form
@@ -963,7 +1230,86 @@ build_volume_list_delta (GList *list_one, GList *list_two)
 
 
 
-#ifdef SOLARIS_MNT
+#ifdef AIX_MNT
+ 
+static GList *
+get_mount_list (NautilusVolumeMonitor *monitor)
+{
+	struct vfs_ent *fs_info;
+	struct vmount *vmount_info;
+	int vmount_number;
+	unsigned int vmount_size;
+	int current;
+	char *device;
+	char *mount;
+	char* fstype;
+	int is_removable;
+	int is_readonly;
+
+	NautilusVolume *volume;
+	GList *volumes = NULL;
+
+	if (mntctl (MCTL_QUERY, sizeof (vmount_size), &vmount_size) != 0) {
+		g_warning ("Unable to know the number of mounted volumes\n");
+
+		return NULL;
+	}
+
+	vmount_info = (struct vmount*)g_malloc (vmount_size);
+	if (vmount_info == NULL) {
+		g_warning ("Unable to allocate memory\n");
+		return NULL;
+	}
+
+	vmount_number = mntctl (MCTL_QUERY, vmount_size, vmount_info);
+
+	if (vmount_info->vmt_revision != VMT_REVISION) {
+		g_warning ("Bad vmount structure revision number, want %d, got %d\n", VMT_REVISION, vmount_info->vmt_revision);
+	}
+
+	if (vmount_number < 0) {
+		g_warning ("Unable to recover mounted volumes information\n");
+
+		g_free (vmount_info);
+		return NULL;
+	}
+
+	while (vmount_number > 0) {
+		device = vmt2dataptr (vmount_info, VMT_OBJECT);
+		mount = vmt2dataptr (vmount_info, VMT_STUB);
+		is_removable = (vmount_info->vmt_flags & MNT_REMOVABLE) ? 1 : 0;
+		is_readonly = (vmount_info->vmt_flags & MNT_READONLY) ? 1 : 0;
+
+		fs_info = getvfsbytype (vmount_info->vmt_gfstype);
+
+		if (fs_info == NULL) {
+			g_warning ("Unknown FS type !\n");
+			fstype = "???";
+		} else {
+			fstype = fs_info->vfsent_name;
+		}
+
+		volume = create_volume (device, mount);
+		volume->is_removable = is_removable;
+		volume->is_read_only = is_readonly;
+
+		volumes = finish_creating_volume_and_prepend
+				(monitor, volume, fstype, volumes);
+
+		/* next entry */
+
+		vmount_info = (struct vmount *)( (char*)vmount_info 
+						+ vmount_info->vmt_length);
+		vmount_number--;
+	}
+	
+	
+	g_free (vmount_info);
+
+        return volumes;
+}
+
+#elif defined(SOLARIS_MNT)
 
 static GList *
 get_mount_list (NautilusVolumeMonitor *monitor) 
@@ -987,12 +1333,12 @@ get_mount_list (NautilusVolumeMonitor *monitor)
 			(monitor, volume, ent.mnt_fstype, volumes);
         }
 
-	fclose (fh);
+	endmntent (fh);
 
         return volumes;
 }
 
-#else /* !SOLARIS_MNT */
+#else /* !AIX_MNT && !SOLARIS_MNT */
 
 static gboolean
 option_list_has_option (const char *optlist,
@@ -1214,10 +1560,7 @@ mount_volumes_check_status (NautilusVolumeMonitor *monitor)
 static int
 get_cdrom_type (const char *vol_dev_path, int* fd)
 {
-#ifndef SOLARIS_MNT
-	*fd = open (vol_dev_path, O_RDONLY|O_NONBLOCK);
-	return ioctl (*fd, CDROM_DISC_STATUS, CDSL_CURRENT);
-#else
+#ifdef SOLARIS_MNT	
 	GString *new_dev_path;
 	struct cdrom_tocentry entry;
 	struct cdrom_tochdr header;
@@ -1254,6 +1597,11 @@ get_cdrom_type (const char *vol_dev_path, int* fd)
 	}
 
 	return type;
+#elif defined(AIX_MNT)
+	return CDS_NO_INFO;
+#else
+	*fd = open (vol_dev_path, O_RDONLY|O_NONBLOCK);
+	return ioctl (*fd, CDROM_DISC_STATUS, CDSL_CURRENT);
 #endif
 }
 
@@ -1768,8 +2116,8 @@ load_additional_mount_list_info (GList *volume_list)
 	for (node = volume_list; node != NULL; node = node->next) {
 		volume = node->data;
 		
-#ifndef SOLARIS_MNT
-		/* These are set up by get_current_mount_list for Solaris. */
+#if !defined(SOLARIS_MNT) && !defined(AIX_MNT)
+		/* These are set up by get_current_mount_list for Solaris&AIX.*/
 		volume->is_removable = volume_is_removable (volume);
 #endif
 		if (volume->volume_name == NULL) {
@@ -1936,6 +2284,29 @@ nautilus_volume_monitor_get_mount_name_for_display (NautilusVolumeMonitor *monit
 	}
 }
 
+/*
+ * HORRORS OF HORROR.
+ * This really should be fixed to not be needed (if it is?).
+ * I just moved it from the desktop icon code so it can be shared
+ * between the desktop icons and the sidebar tree.
+*/
+
+gboolean
+nautilus_volume_is_in_removable_blacklist (const NautilusVolume *volume)
+{
+	char *blacklist[] = { "/proc", "/boot" };
+	int i;
+
+	for (i = 0; i < G_N_ELEMENTS (blacklist); i++) {
+		if (strcmp (blacklist[i], nautilus_volume_get_mount_path (volume)) == 0) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
 #ifdef HAVE_CDDA
 
 static gboolean
