@@ -95,6 +95,7 @@
 #define COMMAND_SHOW_HIDE_TOOLBAR                       "/commands/Show Hide Toolbar"
 #define COMMAND_SHOW_HIDE_LOCATION_BAR                  "/commands/Show Hide Location Bar"
 #define COMMAND_SHOW_HIDE_STATUS_BAR                    "/commands/Show Hide Statusbar"
+#define COMMAND_GO_BURN_CD				"/commands/Go to Burn CD"
 
 #define ID_SHOW_HIDE_SIDEBAR                            "Show Hide Sidebar"
 #define ID_SHOW_HIDE_TOOLBAR                            "Show Hide Toolbar"
@@ -102,6 +103,7 @@
 #define ID_SHOW_HIDE_STATUS_BAR                         "Show Hide Statusbar"
 
 #define START_HERE_URI          "start-here:"
+#define BURN_CD_URI          "burn:"
 
 #define RESPONSE_FORGET		1000
 
@@ -208,6 +210,50 @@ file_menu_close_all_windows_callback (BonoboUIComponent *component,
 	nautilus_application_close_all_windows ();
 }
 
+static void
+file_menu_burn_cd_callback (BonoboUIComponent *component, 
+			    gpointer user_data, 
+			    const char *verb)
+{
+	GError *error;
+	char *argv[] = { "nautilus-cd-burner", NULL};
+	char *text;
+
+	error = NULL;
+	if (!g_spawn_async (NULL,
+			    argv, NULL,
+			    G_SPAWN_SEARCH_PATH,
+			    NULL, NULL,
+			    NULL,
+			    &error)) {
+		text = g_strdup_printf (_("Unable to launch the cd burner application:\n%s"), error->message);
+		eel_show_error_dialog (text,
+				       _("Can't launch cd burner"),
+				       GTK_WINDOW (user_data));
+		g_free (text);
+		g_error_free (error);
+	}
+
+}
+
+static gboolean
+have_burn_uri (void)
+{
+	static gboolean initialized = FALSE;
+	static gboolean res;
+	GnomeVFSURI *uri;
+
+	if (!initialized) {
+		uri = gnome_vfs_uri_new ("burn:///");
+		res = uri != NULL;
+		if (uri != NULL) {
+			gnome_vfs_uri_unref (uri);
+		}
+		initialized = TRUE;
+	}
+	return res;
+}
+
 static void
 nautilus_window_show_location_bar_temporarily (NautilusWindow *window, 
 					       gboolean in_search_mode)
@@ -336,6 +382,15 @@ go_menu_go_to_trash_callback (BonoboUIComponent *component,
 			       EEL_TRASH_URI);
 }
 
+static void
+go_menu_go_to_burn_cd_callback (BonoboUIComponent *component, 
+				gpointer user_data, 
+				const char *verb) 
+{
+	nautilus_window_go_to (NAUTILUS_WINDOW (user_data),
+			       BURN_CD_URI);
+}
+
 static void
 forget_history_if_yes (GtkDialog *dialog, int response, gpointer callback_data)
 {
@@ -710,7 +765,7 @@ help_menu_nautilus_manual_callback (BonoboUIComponent *component,
 		dialog = gtk_message_dialog_new (GTK_WINDOW (window),
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						 _("There was an error displaying help: \n%s"),
 						 error->message);
 		g_signal_connect (G_OBJECT (dialog), "response",
@@ -1169,6 +1224,7 @@ nautilus_window_initialize_menus_part_1 (NautilusWindow *window)
 	BonoboUIVerb verbs [] = {
 		BONOBO_UI_VERB ("New Window", file_menu_new_window_callback),
 		BONOBO_UI_VERB ("Close", file_menu_close_window_callback),
+		BONOBO_UI_VERB ("Burn CD", file_menu_burn_cd_callback),
 		BONOBO_UI_VERB ("Close All Windows", file_menu_close_all_windows_callback),
 #ifdef HAVE_MEDUSA
 		BONOBO_UI_VERB ("Find", file_menu_find_callback),
@@ -1182,6 +1238,7 @@ nautilus_window_initialize_menus_part_1 (NautilusWindow *window)
 		BONOBO_UI_VERB ("Home", go_menu_home_callback),
 		BONOBO_UI_VERB ("Start Here", go_menu_start_here_callback),
 		BONOBO_UI_VERB ("Go to Trash", go_menu_go_to_trash_callback),
+		BONOBO_UI_VERB ("Go to Burn CD", go_menu_go_to_burn_cd_callback),
 		BONOBO_UI_VERB ("Go to Location", go_menu_location_callback),
 		BONOBO_UI_VERB ("Clear History", go_menu_forget_history_callback),
 		BONOBO_UI_VERB ("Reload", view_menu_reload_callback),
@@ -1242,6 +1299,12 @@ nautilus_window_initialize_menus_part_1 (NautilusWindow *window)
 
 	bonobo_ui_component_thaw (window->details->shell_ui, NULL);
 	
+	if (!have_burn_uri ()) {
+		nautilus_bonobo_set_hidden (window->details->shell_ui,
+					    COMMAND_GO_BURN_CD,
+					    TRUE);
+	}
+	
 #ifndef ENABLE_PROFILER
 	nautilus_bonobo_set_hidden (window->details->shell_ui, NAUTILUS_MENU_PATH_PROFILER, TRUE);
 #endif
